/* Pavel Penev, Stara Zagora */

#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define	inp_fpath		"STIR.INP"
#define	outp_fpath		"STIR.OUT"

#define	LM_C_MAX		200

/* Messaging: */
int				gr_err_code;

void sys_err (const char *format, ...)
{
	va_list			va;
	int			orig_errno;

	orig_errno = errno;
	va_start (va, format);
	fprintf (stderr, "\n** ERROR **:\n");
	fprintf (stderr, "\tLast system error: %i %s.\n\t",
		orig_errno, strerror (orig_errno)
	);
	vfprintf (stderr, format, va);
	fprintf (stderr, ".\n");
	va_end (va);
}

void err (const char *format, ...)
{
	va_list			va;

	va_start (va, format);
	fprintf (stderr, "** ERROR **:\n\t");
	vfprintf (stderr, format, va);
	fprintf (stderr, ".\n");
	va_end (va);
}

#ifdef	GRAPHICS

void gr_err (const char *format, ...)
{
	va_list			va;

	va_start (va, format);
	fprintf (stderr, "\n** ERROR **:\n\t");
	vfprintf (stderr, format, va);
	fprintf (stderr, ".\n\tLast graphics error: %i %s.\n",
		gr_err_code, grapherrormsg (gr_err_code));
	va_end (va);
}

#endif /* GRAPHICS */

/* Core: */
typedef	unsigned int		Invs [LM_C_MAX-1];
typedef	unsigned int		Perm [LM_C_MAX];

unsigned int			lm_c;
Invs				invs;
Perm				sol;

int go (void)
{
	unsigned int		i, j;
	unsigned int		num;

	if ( lm_c == 0 ) {
		return (0);
	}
	memset (sol, 0, sizeof (sol [0]) * lm_c);
	num = 1;
	while ( 1 ) {
		i = 0;
		while ( i < lm_c-1 ) {
			if  ( invs [i] == 0 && sol [i] == 0 ) {
				sol [i] = num;
				num++;
				if ( num > lm_c ) {
					return (0);
				}
				for ( j = 0; j < i; j++ ) {
					invs [j]--;
				}
				i = 0;
				continue;
			}
			i++;
		}
		sol [lm_c-1] = num;
		num++;
		if ( num > lm_c ) {
			return (0);
		}
		for ( j = 0; j < lm_c-1; j++ ) {
			invs [j]--;
		}
		i = 0;
	}
	return (0);
}

int init (void)
{
	FILE			*inpf;
	unsigned int		c;

	if ( (inpf = fopen (inp_fpath, "r")) == NULL ) {
		sys_err ("Error opening the input file %s", inp_fpath);
		return (-1);
	}
	if ( fscanf (inpf, "%u\n", &lm_c) != 1 ) {
		sys_err ("Error inputting N");
		fclose (inpf);
		return (-1);
	}
	for ( c = 0; c+1 < lm_c; c++ ) {
		if ( fscanf (inpf, "%u\n", &invs [c]) != 1 ) {
			sys_err ("Error inputtig inversion number %u", c+1);
			fclose (inpf);
			return (-1);
		}
	}
	if ( fclose (inpf) ) {
		sys_err ("Error closing the input file %s", inp_fpath);
		return (-1);
	}
	return (0);
}

int shut (void)
{
	FILE			*outpf;
	unsigned int		c;

	if ( (outpf = fopen (outp_fpath, "w")) == NULL ) {
		sys_err ("Error opening the output file %s", outp_fpath);
		return (-1);
	}
	for ( c = 0; c < lm_c; c++ ) {
		fprintf (outpf, "%u ", sol [c]);
	}
	fprintf (outpf, "\n");
	if ( ferror (outpf) ) {
		sys_err ("Error writing to the output file %s", outp_fpath);
		fclose (outpf);
		return (-1);
	}
	if ( fclose (outpf) ) {
		sys_err ("Error closing the output file %s", outp_fpath);
		return (-1);
	}
	return (0);
}

int main (void)
{
	if ( init () ) {
		return (1);
	}
	if ( go () ) {
		return (2);
	}
	if ( shut () ) {
		return (3);
	}
	return (0);
}