/* Pavel Penev, Stara Zagora */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define	inp_fpath		"SUMS.INP"
#define	outp_fpath		"SUMS.OUT"

#define	N_MAX			30

/* Messaging: */
int				gr_err_code;

void sys_err (const char *format, ...)
{
	va_list			va;
	int			orig_errno;

	orig_errno = errno;
	va_start (va, format);
	fprintf (stderr, "\n** ERROR **:\n");
	fprintf (stderr, "\tLast system error: %i %s.\n\t",
		orig_errno, strerror (orig_errno)
	);
	vfprintf (stderr, format, va);
	fprintf (stderr, ".\n");
	va_end (va);
}

void err (const char *format, ...)
{
	va_list			va;

	va_start (va, format);
	fprintf (stderr, "** ERROR **:\n\t");
	vfprintf (stderr, format, va);
	fprintf (stderr, ".\n");
	va_end (va);
}

#ifdef	GRAPHICS

void gr_err (const char *format, ...)
{
	va_list			va;

	va_start (va, format);
	fprintf (stderr, "\n** ERROR **:\n\t");
	vfprintf (stderr, format, va);
	fprintf (stderr, ".\n\tLast graphics error: %i %s.\n",
		gr_err_code, grapherrormsg (gr_err_code));
	va_end (va);
}

#endif /* GRAPHICS */

/* Core: */
typedef	struct NRepr_ts		NRepr;
typedef	unsigned int		HKey;

struct NRepr_ts {
	unsigned int		SumCnts [N_MAX+1];
	unsigned int		SumCntsC;
};

unsigned int			n;
NRepr				nrepr1, nrepr2;
unsigned int			repr1_end, repr2_end;
int				sol;
NRepr				sol_trace [20*N_MAX];
unsigned int			sol_trace_len;
char				sol_ln_1 [1024],
				sol_ln_last [1024];
unsigned int			reached [N_MAX*N_MAX];

#define	MIN(x,y)		( (x) < (y) ? (x) : (y) )
#define	MAX(x,y)		( (x) > (y) ? (x) : (y) )

int uintcmpfn (const void *a, const void *b)
{
	unsigned int		*i1, *i2;

	i1 = (unsigned int*) a;
	i2 = (unsigned int*) b;
	if ( *i1 > *i2 ) {
		return (+1);
	} else if ( *i1 < *i2 ) {
		return (-1);
	} else {
		return (0);
	}
}

void uint_swap (unsigned int *a, unsigned int *b)
{
	unsigned int		tmp;

	tmp = (*a);
	(*a) = (*b);
	(*b) = tmp;
}

HKey hkey_calc (NRepr *repr)
{
	HKey			key;
	unsigned int		c;

	key = 0;
	for ( c = 0; c < repr->SumCntsC; c++ ) {
		key *= N_MAX;
		key += repr->SumCnts [c];
	}
	return (key);
}

void repr_split_lm (NRepr *repr, unsigned int sum, unsigned int sub_sum)
{
	repr->SumCnts [sum]--;
	repr->SumCnts [sub_sum]++;
	repr->SumCnts [sum - sub_sum]++;
}

void repr_unsplit_lm (NRepr *repr, unsigned int sum, unsigned int sub_sum)
{
	repr->SumCnts [sum - sub_sum]--;
	repr->SumCnts [sub_sum]--;
	repr->SumCnts [sum]++;
}

void repr_sum_lms (NRepr *repr, unsigned int sum1, unsigned int sum2)
{
	repr->SumCnts [sum1]--;
	repr->SumCnts [sum2]--;
	repr->SumCnts [sum1 + sum2]++;
}

void
repr_unsum_lms
(NRepr *repr, unsigned int sum1, unsigned int sum2)
{
	repr->SumCnts [sum1 + sum2]--;
	repr->SumCnts [sum1]++;
	repr->SumCnts [sum2]++;
}

void repr_cp (NRepr *dest, const NRepr *src)
{
	memcpy (dest, src, sizeof (*src));
}

static unsigned int		brepr_lev, brepr_lev_max;

static int brepr_is_sol (void)
{
	unsigned int			subsum;

	for ( subsum = 1; subsum < nrepr1.SumCntsC; subsum++ ) {
		if ( nrepr1.SumCnts [subsum] != nrepr2.SumCnts [subsum] ) {
			return (0);
		}
	}
	return (1);
}

static HKey			brepr_hkey;

int brepr_try (void)
{
	unsigned int		sub_sum;
	unsigned int		try_sum;

	if ( brepr_is_sol () ) {
		sol_trace_len = brepr_lev + 1;
		return (1);
	}
	/*
	brepr_hkey = hkey_calc (&nrepr1);
	if ( reached [brepr_hkey] <= brepr_lev ) {
		return (0);
	}
	reached [brepr_hkey] = brepr_lev;
	if ( brepr_lev >= brepr_lev_max ) {
		return (0);
	}
	*/
	brepr_lev++;
	for ( try_sum = 1; try_sum < nrepr1.SumCntsC; try_sum++ ) {
		if ( nrepr1.SumCnts [try_sum] == 0 ) {
			continue;
		}
		/* Try to split: */
		for ( sub_sum = 1; sub_sum < try_sum; sub_sum++ ) {
			if (	nrepr2.SumCnts [sub_sum] == 0 &&
				nrepr2.SumCnts [try_sum - sub_sum] == 0 &&
				!(
					nrepr2.SumCnts [try_sum] == 0 &&
					nrepr1.SumCnts [try_sum] == 1
				)
			) {
				continue;
			}
			repr_split_lm (&nrepr1, try_sum, sub_sum);
			if ( brepr_try () ) {
				repr_cp (sol_trace + brepr_lev, &nrepr1);
				brepr_lev--;
				repr_unsplit_lm (&nrepr1, try_sum, sub_sum);
				return (1);
			}
			repr_unsplit_lm (&nrepr1, try_sum, sub_sum);
		}
		/* Try to sum: */
		for ( sub_sum = 1; sub_sum < nrepr1.SumCntsC; sub_sum++ ) {
			/* Don't sum with non-existing sub-sum: */
			if ( nrepr1.SumCnts [sub_sum] == 0 ) {
				continue;
			}
			/* If we are trying to double a sub-sum: */
			if (	sub_sum == try_sum &&
				nrepr1.SumCnts [sub_sum] < 2
			) {
				continue;
			}
			if (	nrepr2.SumCnts [try_sum + sub_sum] == 0 &&
				!(
					nrepr2.SumCnts [try_sum] == 0 &&
					nrepr1.SumCnts [try_sum] == 1
				) &&
				!(
					nrepr2.SumCnts [sub_sum] == 0 &&
					nrepr1.SumCnts [sub_sum] == 1
				)
			) {
				continue;
			}
			repr_sum_lms (&nrepr1, sub_sum, try_sum);
			if ( brepr_try () ) {
				repr_cp (sol_trace + brepr_lev, &nrepr1);
				brepr_lev--;
				repr_unsum_lms (&nrepr1, sub_sum, try_sum);
				return (1);
			}
			repr_unsum_lms (&nrepr1, sub_sum, try_sum);
		}
	}
	brepr_lev--;
	return (0);
}

int go (void)
{
	unsigned int		subsum;

	repr_cp (sol_trace + 0, &nrepr1);
	memset (reached, (unsigned int) -1, sizeof (reached));
	for ( brepr_lev_max = 0; ; brepr_lev_max++ ) {
		for ( subsum = 1; subsum < nrepr1.SumCntsC; subsum++ ) {
			if ( nrepr1.SumCnts [subsum] == 0 ) {
				continue;
			}
			brepr_lev = 0;
			if ( brepr_try () ) {
				return (0);
			}
		}
	}
}

int inp_repr (FILE *inpf, unsigned int n, unsigned int repr_n, NRepr *repr)
{
	unsigned int		sum, c, subsum;

	if ( fscanf (inpf, "%u", &subsum) != 1 ) {
		sys_err ("Error inputting term 1 of representation %u",
			repr_n);
		return (-1);
	}
	memset (repr->SumCnts, 0, sizeof (*repr->SumCnts) * (n+1));
	repr->SumCnts [subsum]++;
	repr->SumCntsC = n+1;
	for (	sum = subsum, c = 2;
		sum < n;
		sum += subsum, c++
	) {
		if ( fscanf (inpf, "+%u", &subsum) != 1 ) {
			sys_err ("Error inputting term %u from \
representation %u",
				c, repr_n);
			return (-1);
		}
		repr->SumCnts [subsum]++;
	}
	return (0);
}

void parse_repr (char *inp, NRepr *repr)
{
	unsigned long int	subsum;
	char			*str, *endstr;

	str = inp;
	subsum = strtoul (str, &endstr, 10);
	str = endstr;
	memset (repr->SumCnts, 0, sizeof (*repr->SumCnts) * (n+1));
	repr->SumCnts [subsum]++;
	repr->SumCntsC = n+1;
	while ( *str != '\0' ) {
		/* Skip the '+': */
		str++;
		subsum = strtoul (str, &endstr, 10);
		str = endstr;
		repr->SumCnts [subsum]++;
	}
}

int init (void)
{
	FILE			*inpf;

	if ( (inpf = fopen (inp_fpath, "r")) == NULL ) {
		sys_err ("Error opening the input file %s", inp_fpath);
		return (-1);
	}
	if ( fscanf (inpf, "%u\n", &n) != 1 ) {
		sys_err ("Error inputting N");
		fclose (inpf);
		return (-1);
	}
	/*
	if (	inp_repr (inpf, n, 1, &nrepr1) ||
		inp_repr (inpf, n, 2, &nrepr2)
	) {
		fclose (inpf);
		return (-1);
	}
	*/
	if ( fscanf (inpf, "%s\n%s\n", &sol_ln_1, &sol_ln_last) != 2 ) {
		sys_err ("Error inputtig N's representation");
		fclose (inpf);
		return (-1);
	}
	parse_repr (sol_ln_1, &nrepr1);
	parse_repr (sol_ln_last, &nrepr2);
	if ( fclose (inpf) ) {
		sys_err ("Error closing the input file %s", inp_fpath);
		return (-1);
	}
	return (0);
}

int outp_repr (FILE *outpf, NRepr *repr)
{
	unsigned int			subsum, c;

	subsum = 0;
	while ( 1 ) {
		if ( subsum >= repr->SumCntsC ) {
			return (0);
		}
		if ( repr->SumCnts [subsum] != 0 ) {
			break;
		}
		subsum++;
	}
	fprintf (outpf, "%u", subsum);
	for ( c = 1; c < repr->SumCnts [subsum]; c++ ) {
		fprintf (outpf, "+%u", subsum);
	}
	for ( subsum++; subsum < repr->SumCntsC; subsum++ ) {
		for ( c = 0; c < repr->SumCnts [subsum]; c++ ) {
			fprintf (outpf, "+%u", subsum);
		}
	}
	fprintf (outpf, "\n");
	if ( ferror (outpf) ) {
		sys_err ("Error writing a representation to the output file");
		return (-1);
	}
	return (0);
}

int shut (void)
{
	FILE			*outpf;
	unsigned int		c;

	if ( (outpf = fopen (outp_fpath, "w")) == NULL ) {
		sys_err ("Error opening the output file %s", outp_fpath);
		return (-1);
	}
	fprintf (outpf, "%u\n", sol_trace_len-1);
	if ( sol_trace_len > 1 ) {
		fprintf (outpf, "%s\n", sol_ln_1);
		for ( c = 1; c < sol_trace_len-1; c++ ) {
			if ( outp_repr (outpf, sol_trace + c) ) {
				fclose (outpf);
				return (-1);
			}
		}
		fprintf (outpf, "%s\n", sol_ln_last);
	}
	if ( ferror (outpf) ) {
		sys_err ("Error writing to the output file %s", outp_fpath);
		fclose (outpf);
		return (-1);
	}
	if ( fclose (outpf) ) {
		sys_err ("Error closing the output file %s", outp_fpath);
		return (-1);
	}
	return (0);
}

int main (void)
{
	if ( init () ) {
		return (1);
	}
	if ( go () ) {
		return (2);
	}
	if ( shut () ) {
		return (3);
	}
	return (0);
}