#include <stdio.h>
#include <stdlib.h>

const int MAX_VC = 10000;
const bool DEBUG = false;

FILE *fin;

void infection(const char *);

template <class T>
class List {

	class Node {
		T data;
		Node *next;
		bool bSentinel;

	public:
		Node() {
			next = NULL;
			bSentinel = true;
		}
		Node(T t) {
			data = t;
			next = NULL;
			bSentinel = false;
		}
		~Node() {
			if (next)
				delete next;
		}
		T getData() {
			if (!bSentinel)
				return data;
			else
				infection("Tried to extract value from sentinel list node!\n");
		}
		void setNext(Node *n) {
			next = n;
		}
		Node* getNext() {
			return next;
		}
	};

	Node *root;
	Node *last;
public:

	class Iterator {
		Node *cur;

	public:

		Iterator(Node *c) {
			cur = c;
		}
		bool hasNext() {
			return (cur->getNext() != NULL);
		}
		T getData() {
			return cur->getData();
		}
		void next() {
			if (hasNext())
				cur = cur->getNext();
			else
				infection("Tried to 'next' an non-next-able iterator\n");		 
		}
	};

	List() {
		root = new Node();
		last = root;
	}
	~List() {
		delete root;
	}

	void add(T t) {
		last->setNext(new Node(t));
		last = last->getNext();
	}

	Iterator getIter() {
		return Iterator(root);
	}
};

class Graph {
public:
	class EdgeEnd {
		int iV;
		int iW;
	public:
		EdgeEnd() {
			iV = -1;
			iW = -1;
		}
		EdgeEnd(int iV,int iW) {
			this->iV = iV;
			this->iW = iW;
		}
		int getV() {
			return iV;
		}
		int getW() {
			return iW;
		}
	};
private:
	int iVC;	
	List<EdgeEnd> *adj;

public:
	
	Graph(FILE *f) {
		int iEC;

		fscanf(f,"%d%d",&iVC,&iEC);
		adj = new List<EdgeEnd>[iVC];
		for (int i=0;i<iEC;i++) {
			int iV1,iV2,iW;

			fscanf(f,"%d %d %d",&iV1,&iV2,&iW);
			iV1--; iV2--;
			adj[iV1].add(EdgeEnd(iV2,iW));			
			adj[iV2].add(EdgeEnd(iV1,iW));
		}
	}
	~Graph() {
		delete []adj;
	}

	int getVC() {
		return iVC;
	}

	List<EdgeEnd>& getVertex(int iV) {
		return adj[iV];
	}
};

class HeapNode {
	int iV;
	int iD;
public:
	HeapNode() {
		iV = -1;
		iD = -1;
	}
	HeapNode(int i1,int i2) {
		iV = i1;
		iD = i2;
	}
	int getV() {
		return iV;
	}
	int getD() {
		return iD;
	}

	void setD(int iNewD) {
		iD = iNewD;
	}

	bool operator > (HeapNode& n) {
		return getD() > n.getD();
	}
	bool operator < (HeapNode& n) {
		return getD() < n.getD();
	}
	bool operator >= (HeapNode& n) {
		return getD() <= n.getD();
	}
	bool operator <= (HeapNode& n) {
		return getD() <= n.getD();
	}
};

class Heap {
	HeapNode arr[MAX_VC];	
	int iHeapPos[MAX_VC];
	int iC;

	void heapSwap(int iP1,int iP2) {
		int iP = iHeapPos[arr[iP1].getV()];
		iHeapPos[arr[iP1].getV()] = iHeapPos[arr[iP2].getV()];
		iHeapPos[arr[iP2].getV()] = iP;

		HeapNode h = arr[iP1];
		arr[iP1] = arr[iP2];
		arr[iP2] = h;
	}
	int heapMax(int iP1,int iP2) {
		if (arr[iP1] > arr[iP2])
			return iP1;
		else
			return iP2;
	}
	void fixUp(int iPos) {
		int iP = (iPos - 1) / 2;

		if (arr[iP] < arr[iPos]) {
			heapSwap(iP,iPos);
			fixUp(iP);
		}
	}
	void fixDown(int iPos) {
		int iN1=iPos*2+1,iN2=iPos*2+2;

		if (iN1 >= iC) 
			return ;		
		if (iN2 >= iC) {
			if (arr[iPos] < arr[iN1]) {
				heapSwap(iPos,iN1);
				fixDown(iN1);
			}
		}
		if (arr[iPos] < arr[iN1] &&
			arr[iPos] >= arr[iN2]) {
			heapSwap(iPos,iN1);
			fixDown(iN1);
		}
		if (arr[iPos] >= arr[iN1] &&
			arr[iPos] < arr[iN2]) {
			heapSwap(iPos,iN2);
			fixDown(iN2);
		}
		if (arr[iPos] < arr[iN1] &&
			arr[iPos] < arr[iN2]) {
			int iS = heapMax(iN1,iN2);
			heapSwap(iPos,iS);
			fixDown(iS);
		}
	}

public:

	Heap() {
		iC = 0;
		for (int i=0;i<MAX_VC;i++)
			iHeapPos[i] = -1;
	}
	void push(HeapNode h) {		
		if (h.getV() == 0)
			return;
		forcePush(h);
	}
	void forcePush(HeapNode h) {
		iHeapPos[h.getV()]=iC;

		arr[iC] = h;
		iC++;
		fixUp(iC-1);
	}
	HeapNode pop() {
		iHeapPos[arr[0].getV()]=-2;
		iHeapPos[arr[iC-1].getV()]=0;

		HeapNode h = arr[0];
		arr[0] = arr[iC-1];
		iC--;
		fixDown(0);
		return h;
	}
	void incPrio(int iV,int iNew){
		if (iHeapPos[iV] == -2)
		    return;
		if (iHeapPos[iV] == -1)
			push(HeapNode(iV,iNew));
		else {
			if (iV == 0)
				return;
			if (iNew <= arr[iHeapPos[iV]].getD())
				return;
			arr[iHeapPos[iV]].setD(iNew);
			fixUp(iHeapPos[iV]);
		}
	}
	bool empty() {
		return iC == 0;
	}
};

void infection(const char *msg) {
	if (!DEBUG)
		return;
	fprintf(stderr,"Infection : %s",msg);
	exit(69);
}

int min(int i1,int i2) {
	return i1 < i2 ? i1 : i2;
}

int maxResult(int i1,int i2) {
	if (i1 == 0)
		return i2;
	return i1 > i2 ? i1 : i2;
}

int solve(Graph &g) {
	Heap h;
	int iR = 0;

	h.forcePush(HeapNode(0,0));

	HeapNode v;
	while (!h.empty()) {
		v = h.pop();

        if (v.getV() == g.getVC()-1)
			iR = maxResult(iR,v.getD());

		List<Graph::EdgeEnd> &adj = g.getVertex(v.getV());
		List<Graph::EdgeEnd>::Iterator iter = adj.getIter();

		while (iter.hasNext()) {
			iter.next();
			Graph::EdgeEnd end = iter.getData();		

			if (v.getV() == 0)
				h.incPrio(end.getV(),end.getW());
			else
			    h.incPrio(end.getV(),min(v.getD(),end.getW()));

			//printf("%d %d\n",end.getV(),end.getW());
		}

	}

	return iR;
}

void save(int iR) {
	printf("%d\n",iR);
}

int main() {
	if (DEBUG) 
		fin = fopen("fast.in","r");  //TODO
	else
		fin = stdin;

    Graph *g = new Graph(fin);
	save(solve(*g));
	delete g;

	return 0;
}