/*
x1 x2 x3
y1 y2 y3
 1  1  1

x1*y2 + x2*y3 + x3*y1 - x1*y3 - x2*y1 - x3*y2
x2*y3 - x3*y2
*/

/*
c*c = a*a + b*b - 2*a*b*cos(g)
c*c - a*a - b*b = - 2*a*b*cos(g)
-2*a*b*cos(g) = c*c - a*a - b*b
-cos(g) = (c*c)/(2*a*b) - (a)/(2*b) - (b)/(2*a)
-cos(g) = ((c*c)/(a*b) - a/b - b/a)/2
*/

#include <cmath>
#include <cstdio>
#include <vector>
#include <algorithm>

class point
{
public:
	point(int a = 0, int b = 0) : X(a), Y(b) {}

	int X;
	int Y;
	int ind;
};

inline double get_angle(point, point);

class angle
{
public:
	angle(point a, point b) : A(a), B(b), val(get_angle(a, b)) {}

	point A;
	point B;
	double val;
};

inline bool is_left(point b, point x)
{
	return (b.X*x.Y - x.X*b.Y) > 0;
}

inline double hypot(double a, double b)
{
	return sqrt(a*a + b*b);
}

inline double get_angle(point A, point B)
{
	double a = hypot(B.X, B.Y);
	double b = hypot(A.X, A.Y);
	double c = hypot(std::abs(A.X-B.X), std::abs(A.Y-B.Y));

	return ((c*c)/(a*b) - a/b - b/a) / 2;
}

class pcmp
{
public:
	bool operator() (point b, point x) {return is_left(b, x);}
};

class acmp
{
public:
	bool operator() (angle a, angle b)
	{
		if (a.val != b.val)
			return a.val < b.val;
		else if (a.A.ind != b.A.ind)
			return a.A.ind < b.A.ind;
		else return a.B.ind < b.B.ind;
	}
};

int main()
{
	/* Input */
	int N;
	std::scanf("%d", &N);
	std::vector<point> v(N+1);
	for (int i = 0; i < N+1; i++)
		{std::scanf("%d %d", &(v[i].X), &(v[i].Y)); v[i].ind = i;}

	/* Changing coordinate system */
	for (int i = 1; i < N+1; i++)
		{v[i].X -= v[0].X; v[i].Y -= v[0].Y;}
	v[0].X = 0; v[0].Y = 0;

	/* Sorting points */
	pcmp c;
	std::sort(v.begin(), v.end(), c);

	/* Creating angles */
	std::vector<angle> angles;
	for (int i = 1; i < N; i++)
	{
		if (is_left(v[i], v[i+1]))
			angles.push_back(angle(v[i], v[i+1]));
	}

	if (is_left(v[N], v[1]))
		angles.push_back(angle(v[1], v[N]));

	/* Sorting angles */
	acmp a;
	std::sort(angles.begin(), angles.end(), a);

	/* Output */
	for (int i = 0; i < angles.size(); i++)
		std::printf("%d %d\n", angles[i].A.ind, angles[i].B.ind);


	return 0;
}
