#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define MAXN	5
#define PI		3.1415926535897932384626433832795

typedef struct point
{
	int x, y;
	int i;		// index
} point;

int n;			// points count
point zero;		// zero point
point P[MAXN];	// points array
int A[MAXN];	// angles array

double angleOf(point p)
{
	double a = atan2(p.y, p.x);
	if (a < 0.0)
		a += 2*PI;
	return a;
}

double angleBetween(point p1, point p2)
{
	return fabs(angleOf(p1)-angleOf(p2));
}

double angleOf2(int i)
{
	point p1, p2;
	p1 = P[i];
	if (i+1 != n)
	{
		p2 = P[i+1];
		return angleBetween(p1, p2);
	}
	else
	{
		p2 = P[0];
		return 2*PI - angleBetween(p1, p2);
	}
}

int minOf(int a)
{
	int x = P[a].i;
	int y = P[ (a+1 != n) ? a+1 : 0 ].i;
	return (x < y) ? x : y;
}

int maxOf(int a)
{
	int x = P[a].i;
	int y = P[ (a+1 != n) ? a+1 : 0 ].i;
	return (x > y) ? x : y;
}

int compPoint(point *l, point *r)
{
	double a;
	a = angleOf(*l) - angleOf(*r);
	if (a < 0.0)
		return -1;
	else if (a == 0.0)
		return l->i - r->i;
	else
		return 1;
}

int compAngle(int *l, int *r)
{
	double a1, a2;
	a1 = angleOf2(*l);
	a2 = angleOf2(*r);
	if (a1 < a2)
		return -1;
	else if (a1 == a2)
		return minOf(*l) - minOf(*r);
	else
		return 1;
}

int main(void)
{
	FILE *fin = stdin;
	FILE *fout = stdout;
//	FILE *fin = fopen("rays.in", "r");
	double angle;				// angle between two points
	int i;
	fscanf(fin, "%d %d %d", &n, &zero.x, &zero.y);
	for (i = 0; i < n; i++)
	{
		fscanf(fin, "%d %d", &P[i].x, &P[i].y);
		P[i].x -= zero.x;
		P[i].y -= zero.y;
		P[i].i = i+1;
	}
	qsort(P, n, sizeof(point), compPoint);
	for (i = 0; i < n; i++)
		A[i] = i;
	qsort(A, n, sizeof(int), compAngle);
	for (i = 0; i < n; i++)
	{
		angle = angleOf2(A[i]);
		if (angle > 0.0 && angle < PI && P[A[i]].i == minOf(A[i]))
			fprintf(fout, "%d %d\n", minOf(A[i]), maxOf(A[i]));
	}
	fclose(fin);
	fclose(fout);
	return 0;
}
