#include <stdio.h>
#include <string.h>
#include <stdlib.h>
//#include <time.h>

#define MAXN 1500
#define EPS 1E-12
#define LEFT(a) (2*(a) + 1)
#define RIGHT(a) (2*(a) + 2)
#define PARENT(a) ((a - 1) / 2)
#define COST(a) (pts[(a)].b)
#define INF 0xFFFFFFF
#define MIN(a,b) ((a) < (b) ? (a) : (b))

typedef struct
  {
    double a,b;
  } SPict;

typedef struct
  {
    double w;
    int index;
  } SEnd;

SPict pts[MAXN];
int n;
double w;
SEnd *P[MAXN];
int P_sz[MAXN];
int heap[MAXN];
int heap_sz;
double T1[MAXN];
double T2[MAXN];
double res = INF;

inline void Rotate(int index)
  {
    double tmp = pts[index].a;
    pts[index].a = pts[index].b;
    pts[index].b = tmp;
  }

inline void SiftDown(int index)
  {
    int tmp,t;
    while (1)
      {
        tmp = index;
        if (LEFT(index) < heap_sz && (COST(heap[LEFT(index)]) > COST(heap[tmp])
          || (COST(heap[LEFT(index)]) == COST(heap[tmp]) && pts[heap[LEFT(index)]].a < pts[heap[tmp]].a))) tmp = LEFT(index);
        if (RIGHT(index) < heap_sz && (COST(heap[RIGHT(index)]) > COST(heap[tmp] || (COST(heap[RIGHT(index)]) == COST(heap[tmp]) && pts[heap[RIGHT(index)]].a < pts[heap[tmp]].a)))) tmp = RIGHT(index);

        if (tmp == index) break;
        t = heap[index];
        heap[index] = heap[tmp];
        heap[tmp] = t;

        index = tmp;
      }
  }

inline SiftUp(int index)
  {
    int tmp;
    while (index > 0 && (COST(heap[index]) > COST(heap[PARENT(index)]
      || (COST(heap[index]) == COST(heap[PARENT(index)] && pts[heap[index]].a < pts[heap[PARENT(index)]].a)))))
      {
        tmp = heap[index];
        heap[index] = heap[PARENT(index)];
        heap[PARENT(index)] = tmp;

        index = PARENT(index);
      }
  }

void Init()
  {
    int i;
//    freopen("pict.in","rt",stdin);

    scanf("%d %lf",&n,&w);
    for (i = 0;i < n;i++)
      scanf("%lf%lf",&pts[i].a,&pts[i].b);
  }

void Preprocess()
  {
    int i,j,k;
    double curr_len;

    memset(P_sz,0,sizeof(P_sz));

    for (i = 0;i < n;i++)
      {
        heap_sz = 1;
        heap[0] = i;

        if (pts[i].b > pts[i].a) Rotate(i);
        if (pts[i].a - w > EPS) Rotate(i);
        curr_len = pts[i].a;

        P_sz[i] = 1;
        P[i] = (SEnd*)malloc(sizeof(SEnd));
        P[i][0].w = pts[i].b;
        P[i][0].index = i;

        for (j = i - 1;j >= 0;j--)
          {
            if (pts[j].b > pts[j].a) Rotate(j);

            if (curr_len + pts[j].a <= w + EPS)
              {
                // We can add the current picture
                if (COST(heap[0]) < COST(j))
                  {
                    // Add new interval
                    P_sz[i]++;
                    P[i] = (SEnd*)realloc(P[i],P_sz[i]*sizeof(SEnd));
                    P[i][P_sz[i] - 1].w = COST(j);
                    P[i][P_sz[i] - 1].index = j;
                  }
                else P[i][P_sz[i] - 1].index = j;

                heap[heap_sz] = j;
                SiftUp(heap_sz);
                heap_sz++;
                curr_len += pts[j].a;
              }
            else
              {
                k = 0;
                // Hmmm... there is problem... we must rotate pictures
                while (k < heap_sz && pts[heap[0]].a >= pts[heap[0]].b)
                  {
                    // Try to rotate the current pict
                    if (heap_sz > 1 && COST(heap[1]) > COST(j))
                      {
                        if (heap_sz > 2)
                          {
                             if (COST(heap[2]) > COST(j)) Rotate(j);
                          }
                        else Rotate(j);

                        if (curr_len + pts[j].a <= w + EPS) break;
                      }

                    curr_len -= (pts[heap[0]].a - pts[heap[0]].b);
                    Rotate(heap[0]);
                    SiftDown(0);
                    k++;
                    if (curr_len + pts[j].a <= w + EPS) break;
                  }

                if (!(curr_len + pts[j].a <= w + EPS)) Rotate(j);

                if (curr_len + pts[j].a <= w + EPS)
                  {
                    // We have found good situation
                    heap[heap_sz] = j;
                    SiftUp(heap_sz);
                    heap_sz++;
                    curr_len += pts[j].a;
                    if (COST(heap[0]) - P[i][P_sz[i] - 1].w > EPS)
                      {
                        // Add new interval
                        P_sz[i]++;
                        P[i] = (SEnd*)realloc(P[i],P_sz[i]*sizeof(SEnd));
                        P[i][P_sz[i] - 1].w = COST(heap[0]);
                        P[i][P_sz[i] - 1].index = j;
                      }
                    else P[i][P_sz[i] - 1].index = j;
                  }
                else break;
              }
          }
      }
  }

void Solve()
  {
    double *p1 = T1,*p2 = T2,*tmp;
    int i,j,k;

    for (i = 0;i < n;i++)
      if (P[i][P_sz[i] - 1].index == 0) p1[i] = P[i][P_sz[i] - 1].w;
      else p1[i] = INF;

    res = p1[n - 1];

    // For each level
    for (i = 2;i <= n;i++)
      {
        // For each picture to be the last one
        for (j = 0;j < n;j++)
          {
            p2[j] = INF;
            // For each interval we can make
            for (k = 0;k < P_sz[j];k++)
              {
                if (p2[j] < P[j][k].w) break; // No chance for better solution
                if (P[j][k].index >= i - 1) p2[j] = MIN(p2[j],p1[P[j][k].index - 1] + P[j][k].w);
                else break;
              }
          }
        if (p2[n - 1] < res) res = p2[n - 1];

        // Swap pointers
        tmp = p1;
        p1 = p2;
        p2 = tmp;
      }
  }

void Output()
  {
    printf("%.6lf\n",res);
  }

int main()
  {
//    clock();
    Init();
    Preprocess();
    Solve();
    Output();
//    printf("Time : %lf sec.\n",(double)clock() / CLOCKS_PER_SEC);
    return 0;
  }
