// unit totolib.
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "totolib.h"

#define MAX_TRY 45
#define MAX_SCORE 10

static int iiisol[6], bbbsol[50];
static int try_rem = MAX_TRY;
static int inited=0;
FILE *l0g = NULL;

static void bailout(char *msg, int score)
{
       fprintf(l0g, "%s\n%d\n", msg, score);
       // print magic:
       fprintf(l0g, "---\n");
       fclose(l0g);
       exit(0);
}

static int formula(int tries)
{
       if (tries <= 28) return MAX_SCORE;
       if (tries <= 31) return  80 * MAX_SCORE / 100;
       if (tries <= 35) return  60 * MAX_SCORE / 100;
       if (tries <= 40) return  40 * MAX_SCORE / 100;
       if (tries <= 45) return  20 * MAX_SCORE / 100;
       return 0;
}

static int test_fish(int *a)
{
    int i,j;
    for (i=0;i<6;i++)
        if (a[i]<1 || a[i]>49) return 0;
    for (i=1;i<6;i++)
        for (j=0;j<i;j++)
	        if (a[i]==a[j]) return 0;
     return 1;
}

static int input_from_file(char *fn, int *r)
{
    FILE *f;
    f = fopen(fn, "rt");
    if (f==NULL) {
        fprintf(l0g, "Can't open file `%s'; switching to random-generation.\n",fn);
        return 0;
    }
    if (fscanf(f,"%d%d%d%d%d%d",r,r+1,r+2,r+3,r+4,r+5)!=6) {
        fprintf(l0g, "Can't read 6 ints from input file!\n");
        return 0;
    }
    fclose(f);
    if (!test_fish(r)) {
        fprintf(l0g, "Given combination is bad! Switching to random-generation.\n");
        return 0;
    }
    fprintf(l0g, "Data from `%s' read successfully.\n",fn);
    return 1;
}

static void rand_fish(int *r)
{
    int i;
    do {
        for (i=0;i<6;i++) r[i] = rand()%49 + 1;
    } while (!test_fish(r));
}

static void init(void)
{
    int a[6], i;
    if (inited) {
        printf("This shouldn't happened. Somebody called init()!\n");
        exit(1);
    }
    inited = 1;
    l0g = fopen("toto.log", "wt");
    if (l0g==NULL) {
        printf("Cannot open log file. Sorry.\n");
        exit(1);
    }    
    if (!input_from_file("toto.in", a)) {
	    srand(~((unsigned int) time(NULL)));
	    rand_fish(a);
    }
    for (i=0;i<49;i++) bbbsol[i]=0;    
    for (i=0;i<6;i++) {
	       iiisol[i]=a[i];     
	       bbbsol[a[i]]=1;
    }
#ifndef secure
    fprintf(l0g, "Using combination (%d %d %d %d %d %d) as target.\n", a[0], a[1], a[2], a[3], a[4], a[5]);
#endif
}

int post_comb(int v1, int v2, int v3, int v4, int v5, int v6)
{
    int i, a[6], r;
    if (!inited) {
        init();
    }
    a[0]=v1; a[1]=v2; a[2]=v3; a[3]=v4; a[4]=v5; a[5]=v6;
    fprintf(l0g,"post_comb(%d, %d, %d, %d, %d, %d) : ",v1,v2,v3,v4,v5,v6);
    if (!test_fish(a)) {
        fprintf(l0g, "Bad Combination!!!\n");
        bailout("WRONG COMB", 0.0);
    }
    if (!try_rem) {
        fprintf(l0g, "Max tries exceeded.\n");
        bailout("LIMIT", 0.0);
    }
    try_rem--;
    r = 0;
    for (i=0;i<6;i++)
        if (bbbsol[a[i]]) r++;
    if (r < 6) {
        fprintf(l0g, "%d (query no. %d)\n", r, MAX_TRY-try_rem);
        return r;
    } else {
        fprintf(l0g, "6 - SUCCESS\n\nAnswered correctly after %d queries.\n", MAX_TRY-try_rem);
        bailout("OK", formula(MAX_TRY-try_rem));
    }
    return 0; //dummy
}

