/*
KEYW: geometry
*/

#include <cstdio>
#include <cmath>
#include <algorithm>

const int MAXN = 1 << 14;
const int MAXHLEN = 1 << 15;
const double _infinity = 1E+10;

typedef long long lli;

struct point {
	int x;
	int y;
	point (int xx =0, int yy =0) : x (xx), y (yy) {}
	point (const point &a) : x (a.x), y (a.y) {}
};

struct el {
	int to;
	double dist;
	el *next;
};

struct heap {
	int to;
	double dist;
	heap (int tt =0, double dd =0.0) : to (tt), dist (dd) {}
};

int N, M;
int NM;

heap h[MAXHLEN];
int hlen;
int idh[MAXN * 2];
const int NOT_IN = -1;
const int POPPED = -2;

inline bool cmp_heap (const heap &a, const heap &b);

void init () {
	int i;
	for (i = 1; i < MAXHLEN; ++i)
		h[i].dist = _infinity;
	h[0].dist = -_infinity;

	hlen = 1;

	for (i = 0; i < NM; ++i)
		idh[i] = NOT_IN;
}

void siftup (int a) {
	heap tomove = h[idh[a]];

	int p1 = idh[a], p2 = p1 / 2;

	while (
//	h[p2].dist > tomove.dist
	cmp_heap (tomove, h[p2])
	) {
		h[p1] = h[p2];
		idh[h[p2].to] = p1;
		p1 = p2;
		p2 /= 2;
	}
	idh[a] = p1;
	h[p1] = tomove;
}

void add (int el, double dist) {
	if (idh[el] == NOT_IN) {
		h[hlen].to = el;
		h[hlen].dist = dist;
		idh[el] = hlen++;
		siftup (el);
	} else if (idh[el] != POPPED &&
//	h[idh[el]].dist > dist
	cmp_heap (heap (el, dist), h[idh[el]])
	) {
		h[idh[el]].dist = dist;
		siftup (el);
	}
}

heap pop () {
	heap ret = h[1];
	idh[ret.to] = POPPED;
	heap tomove = h[--hlen];
	h[hlen].dist = _infinity;

	if (hlen == 1)
		return ret;

	int p1 = 1, p2 = 2;

	for (;;) {
		if (
//		h[p2+1].dist < h[p2].dist
		cmp_heap (h[p2+1], h[p2])
		)
			++p2;
		if (
//		h[p2].dist < tomove.dist
		cmp_heap (h[p2], tomove)
		) {
			h[p1] = h[p2];
			idh[h[p2].to] = p1;
			p1 = p2;
			p2 *= 2;
		} else
			break;
	}

	h[p1] = tomove;
	idh[tomove.to] = p1;

	return ret;
}

el *vec[MAXN * 2];



double dijkstra () {
	init ();
	add (0, 0.0);
	heap crnt;

	while (hlen > 1) {
		crnt = pop ();

		if (crnt.to == NM-1) return crnt.dist;

		for (el *k = vec[crnt.to]; k; k = k->next)
			add (k->to, k->dist + crnt.dist);
	}
	return -1.0;
}

point data[MAXN * 2];
int indexx[MAXN * 2];
double _dist_END [MAXN * 2];

inline bool cmp_heap (const heap &a, const heap &b) {
	return (a.dist + _dist_END [a.to] < b.dist + _dist_END [b.to]);
}

inline int sqr (int a) {
	return a * a;
}

inline double _dist (point a, point b) {
	return sqrt (sqr (a.x - b.x) + sqr (a.y - b.y));
}

lli S (point a, point b, point c) {
	return ((lli)a.x * b.y + (lli)b.x * c.y + (lli)c.x * a.y - (lli)a.x * c.y - (lli)c.x * b.y - (lli)b.x * a.y);
}

el* getnew () {
	static el* buf = 0;
	static int crnt = 10000;

	if (crnt == 10000) {
		buf = new el [10000];
		crnt = 0;
	}
	return &buf[crnt++];
//	return new el;
}

void findpaths (int a) {
	point lb(data[indexx[a]].x - 1, data[indexx[a]].y);
	point rb(data[indexx[a]].x + 1, data[indexx[a]].y);
	point base(data[indexx[a]]);
	el *tmp;

//	printf ("find paths - %d\n", a);
	for (int i = a + 1; i < NM; ++i) {
//		printf ("i == %d\n", i);
		if (indexx[i] < N) {//left bound
			if (S (base, lb, data[indexx[i]]) > 0) continue;//lefter than the left bound
			if (S (base, rb, data[indexx[i]]) < 0) break;//righter than the right bound
			lb = data[indexx[i]];//new left bound
//			printf ("adding\n");
			//add in vec;
			tmp = getnew ();
			tmp->to = indexx[i];
			tmp->dist = _dist (base, data[indexx[i]]);
			tmp->next = vec[indexx[a]];
			vec[indexx[a]] = tmp;
			//end add in vec
		} else {//right bound
			if (S (base, rb, data[indexx[i]]) < 0) continue;//righter than the right bound
			if (S (base, lb, data[indexx[i]]) > 0) break;//lefter than the left bound
			rb = data[indexx[i]];//new left bound
//			printf ("adding\n");
			//add in vec;
			tmp = getnew ();
			tmp->to = indexx[i];
			tmp->dist = _dist (base, data[indexx[i]]);
			tmp->next = vec[indexx[a]];
			vec[indexx[a]] = tmp;
			//end add in vec
		}
	}
}

bool cmp (const int a, const int b) {
	if (data[a].y != data[b].y)
		return data[a].y < data[b].y;
	else
		return data[a].x < data[b].x;
}

int main () {
	freopen ("error", "w", stderr);

	scanf ("%d %d", &N, &M);
	NM = M + N;

	int i;
	for (i = 0; i < N; ++i)
		scanf ("%d %d", &data[i].x, &data[i].y);

	for (i = N; i < NM; ++i)
		scanf ("%d %d", &data[i].x, &data[i].y);

	for (i = 0; i < NM; ++i)
		_dist_END[i] = _dist (data[i], data[NM-1]);

	for (i = 0; i < NM; ++i) indexx[i] = i;
	std::sort (indexx, indexx + NM, cmp);
/*
	for (i = 0; i < NM; ++i)
		printf ("%d ", indexx[i]);
	printf ("\n");

	printf ("befor finding!\n");
*/
	for (i = 0; i < NM; ++i)
		findpaths (i);
/*
	for (i = 0; i < NM; ++i) {
		printf ("%d ->", i);
		for (el *tmp = vec[i]; tmp; tmp = tmp->next)
			printf (" (%d %lf)", tmp->to, tmp->dist);
		printf ("\n");
	}
*/
//	printf ("before dijkstra!\n");

	double dist = dijkstra ();

	printf ("%.4lf\n", dist);

	return 0;
}
