/*
Algoritmul forta bruta + cache

Complexitate timp: O(m^2*n)
Complexitate spatiu: O(m), netinand cont de spatiul ocupat de matrice
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "comdef.h"

#define SWAP(x,y) (x^=(y^=(x^=y)))
#define AREA(llx,lly,urx,ury) (llx>urx || lly>ury)?0:(urx-llx+1)*(ury-lly+1)
#define MIN(a,b) ((a)>(b))?(b):(a)

#define MAX 700

typedef u8 *vector;

vector b[MAX];
i16 c[MAX];
i16 m,n;
i32 maxArea=0;
boolean swapped;

void Read()
{
    i16 aux;
    i16 x,y;

    FILE *fin=fopen("matrix.in","rt");

    fscanf(fin,"%hd%hd",&m,&n);

    if (m>n)
    //Deoarece complexitatea algoritmului este O(M^2*N), daca avem M>N transpunem matricea si rezolvam problema pe transpusa, solutia problemei obtinute astfel fiind simetrica solutiei matricei netranspuse. Avantajul il consta complexitatea timp care devine O(M*N^2)
    {
        swapped=true;
        for (y=0; y<n; y++)
            b[y]=(vector)malloc(sizeof(u8)*m);                

        for (x=0; x<m; x++)
            for (y=0; y<n; y++)
            {
                fscanf(fin,"%hd",&aux);
                b[y][x]=(u8)aux;
            }
        
        SWAP(m,n);
    }
    else
    {
        swapped=false;
        for (y=0; y<m; y++)
            b[y]=(vector)malloc(sizeof(u8)*n);                

        for (y=0; y<m; y++)
            for (x=0; x<n; x++)
            {
                fscanf(fin,"%hd",&aux);
                b[y][x]=(u8)aux;
            }
    }
    
    fclose(fin); 
}

void GrowOnes(i32 llx, i32 lly, i32 &urx, i32 &ury)
{
    i32 xMax;
    i32 y;
    i32 localMaxArea=0;

    urx=llx-1;
    ury=lly-1;
    
    xMax=n-1;
    for (y=lly; y<m; y++)
    {
        if (b[y][llx]==0)
            break;

        xMax=MIN(llx+c[y]-1,xMax);
        if (AREA(llx,lly,xMax,y)>localMaxArea)
        {
            localMaxArea=AREA(llx,lly,xMax,y);
            urx=xMax;
            ury=y;
        }
    }
}

void UpdateCache(i32 x)
{
    for (i32 y=0; y<m; y++)
        if (b[y][x]==1)
            c[y]++;
        else
            c[y]=0;
}

void Solve()
{
    i32 best_llx=0,best_lly=0,
        best_urx=-1,best_ury=-1;
    i32 llx,lly,urx,ury; //ll=lower-left, ur=upper-right (where (0,0) is lower and to the left of (1,1) )

    memset(c,0,sizeof(i16)*m);
    maxArea=0;

    for (llx=n-1; llx>=0; llx--)
    {
        UpdateCache(llx);
        for (lly=0; lly<m; lly++)
        {
            GrowOnes(llx,lly,urx,ury);
            if (AREA(llx,lly,urx,ury)>maxArea)
            {
                best_llx=llx;
                best_lly=lly;
                best_urx=urx;
                best_ury=ury;
                maxArea=AREA(best_llx,best_lly,best_urx,best_ury);
            }
        }
    }

    /*
    if (swapped)
        printf("Swapped: (%ld,%ld)->(%ld,%ld)\n",best_llx,best_lly,best_urx,best_ury);
    else
        printf("(%ld,%ld)->(%ld,%ld)\n",best_lly,best_llx,best_ury,best_urx);
    */

    freopen("matrix.out","wt",stdout);
    printf("%ld",maxArea);
}

void main()
{
    Read();
    Solve();

    i16 y;

    if (swapped)
        for (y=0; y<n; y++)
            free(b[y]);
    else
        for (y=0; y<m; y++)
            free(b[y]);
}