/*
Program: problema 25 (pacman)
Autor: Cristi Francu, sursa neoficiala

Comentarii: se bazeaza pe functia standard "clock" prezenta in
<time.h>. Functioneaza sub unix, nu pot sa-l incerc sub DOS :-( De
asemenea, este extrem de important ca programul sa fie compilat cu
grad maxim de optimizari, si nu stiu ce directive pragma trebuie sa
adaug pentru asta.

Algoritmi folositi: programul lanseaza mai intii o cautare greedy. Se
selecteaza in permanenta urmatoarea cireasa ca fiind cea mai
apropiata.  Aceasta este prima cale generata. Urmatoarele cai sint
generate absolut aleator. Asupra tuturor cailor se aplica operatori de
ridicare a numarului de cirese colectate, daca este posibil. Dupa ce
caile sint ridicate la maxim in urma operatorilor, cea mai buna cale
este in mod evident afisata.

Tehnica este generala, si se numeste "hill-climbing".  Ideea e ca daca
ai un munte si vrei sa-i afli maximul, faci asa: pornesti dintr-un
punct aleator si aplici operatori de ridicare a inaltimii pina ce nu
mai poti sa urci. In acel moment ai gasit un punct de maxim
local. Apoi pornesti din alt punct aleator si tot asa. Cu cit mai
multe puncte cu atit mai mare sansa sa gasesti maximul global, sau
macar un maxim cit mai apropiat de maximul global.

In cazul acestui program, caile sint punctele de pornire, si am
definit trei operatori de ridicare a caii: pull, flex si stretch.

"Pull" observa daca exista o intoarcere de genul EV sau SN si daca nu se
trece printr-o cireasa care era luata in acel fel, atunci "intinde"
calea, practic lungind-o cu doua casute. Cele doua casute la coada se
adauga greedy, astfel incit sa se colecteze cit mai multe cirese.

"Flex" observa daca exista o cotitura la 90 grade. In patratul definit
de aceasta cotitura ramine o casuta neatinsa. Daca in ea exista o cireasa
nemincata, curba se inverseaza astfel incit sa treaca prin ea, bine-nteles
daca nu cumva se pierde o cireasa in felul asta.

"Stretch" este cumva opusul lui "pull". Daca o casuta a caii trece pe
linga o cireasa, atunci calea incearca sa se intinda in directia
ciresei, astfel incit sa o inglobeze. Calea este scurtata cu
2. Desigur operatorul se aplica numai atunci cind este convenabil.

Mai multe detalii la francu@cs.rutgers.edu (remarcati schimbarea de
adresa pentru cei ce ma cunoc deja)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <time.h>

#define DATAIN "PACMAN.IN"
#define DATAOUT "PACMAN.OUT"
#define QLEN 16384
#define TIMES 1000
/* #define DEBUG */

int M, N, T, Plin, Pcol, cherry_, step_, maxstep_;
char **maze, **mazeCopy;
unsigned char **visited;
char *path, *maxpath, *pathlin, *pathcol;
char delta[4][3] = { {-1, 0, 'N'}, {1, 0, 'S'}, {0, -1, 'V'}, {0, 1, 'E'} }; 
char toDir[3][3] = { {'*', 'N', '*'}, {'V', '*', 'E'}, {'*', 'S', '*'} } ;
int goOn_;
long clock_;

#define checkTime if(!(goOn_ = (((currentClock - clock_) / CLOCKS_PER_SEC) < 4))) return

void readMaze()
{
  int i, j;
  FILE *in = fopen(DATAIN, "r");
  if(in == NULL)
    {
      fprintf(stderr, "file not found: %s\n", DATAIN);
      exit(1);
    }
  fscanf(in, "%d%d%d", &M, &N, &T);
  maze = malloc(M * sizeof(char *));
  mazeCopy = malloc(M * sizeof(char *));
#ifdef DEBUG
  assert(maze != NULL);
#endif  
  visited = malloc(M * sizeof(char *));
#ifdef DEBUG
  assert(visited != NULL);
#endif  

  for(i = 0; i < M; i++)
    {
      maze[i] = malloc(2 + N * sizeof(char));
      mazeCopy[i] = malloc(2 + N * sizeof(char));
#ifdef DEBUG
      assert(maze[i] != NULL);
#endif  
      fscanf(in, "%s", maze[i]);
      maze[i][N] = 0;
      strcpy(mazeCopy[i], maze[i]);
      visited[i] = malloc(N * sizeof(char));
#ifdef DEBUG
      assert(visited[i] != NULL);
#endif  
      for(j = 0; j < N; j++)
	visited[i][j] = 0;
    }
  fclose(in);
  path = malloc(1 + T * sizeof(char));
#ifdef DEBUG
  assert(path != NULL);
#endif  
  path[T] = 0;
  maxpath = malloc(1 + T * sizeof(char));
#ifdef DEBUG
  assert(maxpath != NULL);
#endif  
  maxpath[T] = 0;
  pathlin = malloc(1 + T * sizeof(char));
#ifdef DEBUG
  assert(pathlin != NULL);
#endif  
  pathcol = malloc(1 + T * sizeof(char));
#ifdef DEBUG
  assert(pathcol != NULL);
#endif  

#ifdef DEBUG
  fprintf(stderr, "Finished allocating memory\n");
  fprintf(stderr, "T=%d\n", T);
#endif
}

void printMaze()
{
  int i;
  for(i = 0; i < M; i++)
    {
      printf("%s\n", maze[i]);
    }
}

int check(int lin, int col)
{
  if((lin < 0) || (lin >= M)) return 0;
  if((col < 0) || (col >= N)) return 0;
  return maze[lin][col] != '*';
}


int generatePath(int i, int cherry, int Plin, int Pcol)
{
  int succeeded, lin = Plin, col = Pcol, dlin, dcol;
  int angle;
  char dir;
  
#ifdef DEBUG
  fprintf(stderr, "entered generatePath\n");
#endif
  
  for(; i < T; i++)
    {
      succeeded = 0;
      while(!succeeded)
	{
	  angle = rand() % 4;
	  succeeded = check(lin + delta[angle][0], col + delta[angle][1]);
	}
      pathlin[i+1] = (lin += delta[angle][0]);
      pathcol[i+1] = (col += delta[angle][1]);
#ifdef DEBUG
      fprintf(stderr, "generated <%d, %d>\n", lin, col);
#endif
  
      if((maze[lin][col] == '%') && !visited[lin][col])
	cherry++;
      visited[lin][col]++;
    }
  return cherry_ = cherry;
}

void cleanPath()
{
  int i;
  for(i = 0; i < T; i++)
    visited[pathlin[i+1]][pathcol[i+1]] = 0;
}

void generateMoves()
{
  int i;
  for(i = 0; i < T; i++)
      path[i] = toDir[pathlin[i+1]-pathlin[i]+1][pathcol[i+1]-pathcol[i]+1];
}

int checkFlex(int i, int *lin, int *col)
{
  if((maze[pathlin[i]][pathcol[i]] == '%') &&
     (visited[pathlin[i]][pathcol[i]] == 1)) /* can't increase */
      return 0;
      
  if(i == T) /* last cell, special case */
    {
      *lin = pathlin[i-1];
      *col = pathcol[i-1] + 1;
      if(check(*lin, *col)) /* possible to move here? */
	{
	  if((!visited[*lin][*col]) &&
	     (maze[*lin][*col] == '%'))
	      return 1;
	}

      *lin = pathlin[i-1];
      *col = pathcol[i-1] - 1;
      if(check(*lin, *col)) /* possible to move here? */
	{
	  if((!visited[*lin][*col]) &&
	     (maze[*lin][*col] == '%'))
	      return 1;
	}

      *lin = pathlin[i-1] + 1;
      *col = pathcol[i-1];
      if(check(*lin, *col)) /* possible to move here? */
	{
	  if((!visited[*lin][*col]) &&
	     (maze[*lin][*col] == '%'))
	      return 1;
	}

      *lin = pathlin[i-1] - 1;
      *col = pathcol[i-1];
      if(check(*lin, *col)) /* possible to move here? */
	{
	  if((!visited[*lin][*col]) &&
	     (maze[*lin][*col] == '%'))
	      return 1;
	}

      return 0;
    }
  
  if((abs(pathlin[i-1] - pathlin[i+1]) == 1) &&
     (abs(pathcol[i-1] - pathcol[i+1]) == 1))
    {
      if(pathlin[i] == pathlin[i - 1])
	{
	  (*lin) = pathlin[i + 1];
	  (*col) = pathcol[i - 1];
	}
      else
	{
	  (*lin) = pathlin[i - 1];
	  (*col) = pathcol[i + 1];
	}
      if((!visited[*lin][*col]) &&
	 (maze[*lin][*col] == '%'))
	return 1;
    }
  return 0;
}

int checkStretch(int i, int *lin, int *col)
{
  if((maze[pathlin[T]][pathcol[T]] == '%') &&
     (visited[pathlin[T]][pathcol[T]] == 1)) /* can't increase */
      return 0;
  if(i <= (T-2))
    if((maze[pathlin[T-1]][pathcol[T-1]] == '%') &&
     (visited[pathlin[T-1]][pathcol[T-1]] == 1)) /* can't increase */
          return 0;
  
  *lin = pathlin[i];
  *col = pathcol[i] - 1;
  if(check(*lin, *col)) /* possible to move here? */
    {
      if((!visited[*lin][*col]) &&
	 (maze[*lin][*col] == '%'))
	return 1;
    }

  *lin = pathlin[i];
  *col = pathcol[i] + 1;
  if(check(*lin, *col)) /* possible to move here? */
    {
      if((!visited[*lin][*col]) &&
	 (maze[*lin][*col] == '%'))
	return 1;
    }

  *lin = pathlin[i] - 1;
  *col = pathcol[i];
  if(check(*lin, *col)) /* possible to move here? */
    {
      if((!visited[*lin][*col]) &&
	 (maze[*lin][*col] == '%'))
	return 1;
    }

  *lin = pathlin[i] + 1;
  *col = pathcol[i];
  if(check(*lin, *col)) /* possible to move here? */
    {
      if((!visited[*lin][*col]) &&
	 (maze[*lin][*col] == '%'))
	return 1;
    }

  return 0;
}

int checkPull(int i, int *plin1, int *pcol1, int *plin2, int *pcol2, int *gain)
{
  int k, j, lin1, col1, lin2, col2;
  int localGain, maxGain;
  
  if(i > (T - 2)) /* not enough moves left to make the pull */
    return 0;
  if((pathlin[i] != pathlin[i+2]) || (pathcol[i] != pathcol[i+2]))
    return 0; /* not a pull loop */
  *gain = 0;

  if((maze[pathlin[i+1]][pathcol[i+1]] == '%') &&
     (visited[pathlin[i+1]][pathcol[i+1]] == 1)) /* lose one cherry */
      (*gain)--;

  /* trick the greedy search by pretending this square is not
     visited by the i+1 move anymore */
  visited[pathlin[i+1]][pathcol[i+1]]--;
  
  maxGain = 0;
  for(k = 0; k < 4; k++)
    {
      lin1 = pathlin[T] + delta[k][0];
      col1 = pathcol[T] + delta[k][1];
      if(check(lin1, col1))
	for(j = 0; j < 4; j++)
	  {
	    lin2 = lin1 + delta[j][0];
	    col2 = col1 + delta[j][1];
	    if(check(lin2, col2)) /* found a path of length 2 */
	      {
		localGain = 0;
		if((maze[lin1][col1] == '%') &&
		   (!visited[lin1][col1]))
		  localGain++;
		if((maze[lin2][col2] == '%') &&
		   (!visited[lin2][col2]))
		  localGain++;
		if(localGain > maxGain) /* found a better path of length 2 */
		  {
		    *plin1 = lin1;
		    *pcol1 = col1;
		    *plin2 = lin2;
		    *pcol2 = col2;
		    maxGain = localGain;
		  }
	      }
	    
	  }
    }
  /* restore the old state of this cell */
  visited[pathlin[i+1]][pathcol[i+1]]++;
  return (((*gain) += maxGain) > 0);
}
  
void climbPath()
{
  int i, j, changed;
  long currentClock;
  
  int auxlin, auxcol;
  int auxlin1, auxcol1, auxlin2, auxcol2, gain;
  step_ = 0;
  do
    {
      changed = 0;
      for(i = 0; i < T; i++)
	{
	  currentClock = clock();

	  checkTime;

	  /* try a pull */
	  if(checkPull(i, &auxlin1, &auxcol1, &auxlin2, &auxcol2, &gain))
	    {
#ifdef DEBUG
	      fprintf(stderr, "pulling move number %d\n", i);
	      generateMoves();
	      fprintf(stderr, "old path: <%s>\n", path);
#endif
	      ++step_;
	      changed = 1;
	      visited[pathlin[i+1]][pathcol[i+1]]--;
	      visited[pathlin[i+2]][pathcol[i+2]]--;
	      for(j = i + 1; j < (T - 1); j++)
		{
		  pathlin[j] = pathlin[j + 2];
		  pathcol[j] = pathcol[j + 2];
		}
	      pathlin[T - 1] = auxlin1;
	      pathcol[T - 1] = auxcol1;
	      pathlin[T] = auxlin2;
	      pathcol[T] = auxcol2;

	      visited[pathlin[T - 1]][pathcol[T - 1]]++;
	      visited[pathlin[T]][pathcol[T]]++;

	      cherry_ += gain;
#ifdef DEBUG
	      generateMoves();
	      fprintf(stderr, "new path: <%s>\n", path);
#endif
	    }
      
	  checkTime;

	  /* try a flex */
	  if(checkFlex(i + 1, &auxlin, &auxcol))
	    { 
#ifdef DEBUG
	      fprintf(stderr, "flexing move number %d\n", i + 1);
	      generateMoves();
	      fprintf(stderr, "old path: <%s>\n", path);
#endif
	      ++step_;
	      changed = 1;
	      visited[pathlin[i+1]][pathcol[i+1]]--;
	      pathlin[i + 1] = auxlin;
	      pathcol[i + 1] = auxcol;
	      visited[pathlin[i+1]][pathcol[i+1]]++;
	      cherry_++; 
#ifdef DEBUG
	      generateMoves();
	      fprintf(stderr, "new path: <%s>\n", path);
#endif
	    }
      

	  checkTime;
	  /* try a stretch */
	  if(checkStretch(i, &auxlin, &auxcol))
	    {
#ifdef DEBUG
	      fprintf(stderr, "stretching move number %d\n", i);
	      generateMoves();
	      fprintf(stderr, "old path: <%s>\n", path);
#endif
	      ++step_;
	      changed = 1;
	      visited[pathlin[T]][pathcol[T]]--;
	      if(i <= (T - 2))
		visited[pathlin[T - 1]][pathcol[T - 1]]--;
	      for(j = T; j > (i + 1); j--)
		{
		  pathlin[j] = pathlin[j - 2];
		  pathcol[j] = pathcol[j - 2];
		}
	      pathlin[i + 1] = auxlin;
	      pathcol[i + 1] = auxcol;

	      visited[pathlin[i + 1]][pathcol[i + 1]]++;
	      if(i <= (T - 2))
		visited[pathlin[i + 2]][pathcol[i + 2]]++;

	      cherry_++;
#ifdef DEBUG
	      generateMoves();
	      fprintf(stderr, "new path: <%s>\n", path);
#endif
	    }

	  
/*	  if(!changed) */
	}
    }
  while(changed);
  
#ifdef DEBUG
  if(step_)
    fprintf(stderr, "done %d climbs\n", step_);
#endif
}

void findP()
{
  int i, j;
  for(i = 0; i < M; i++)
    for(j = 0; j < N; j++)
      if(maze[i][j] == 'P')
	{
	  pathlin[0] = Plin = i;
	  pathcol[0] = Pcol = j;
	}
}
  
void printPath(char *path, int maxcherry)
{
  FILE *out = fopen(DATAOUT, "w");
  int i;
  
  if(out == NULL)
    fprintf(stderr, "couldn't open %s for writing!\n", DATAOUT);
#ifdef DEBUG
  if(path == NULL)
    fprintf(stderr, "null path!\n");
  fprintf(stderr, "writing moves\n");
#endif
  for(i = 0; path[i]; i++);
  fprintf(stderr, "path length=%d\n", i);

  fprintf(out, path);
  fprintf(out, "\n%d\n", maxcherry);
  fclose(out);
}

int findNearestCherry(int lin, int col, char *pLin, char *pCol)
{
  int i, j, n = 0, qfirst = 0, qlast = 1, clin, ccol, goOn, auxlin;
  char prevLin[100][100], prevCol[100][100];
  char qlin[QLEN], qcol[QLEN];
  
  qlin[0] = lin;
  qcol[0] = col;
  
  prevLin[lin][col] = prevCol[lin][col] = -1;
  visited[lin][col] = 1;
  
  goOn = 1;
  while((qfirst != qlast) && goOn)
    {
      clin = qlin[qfirst];
      ccol = qcol[qfirst++];
#ifdef DEBUG
      fprintf(stderr, "processing cell <%d, %d>\n", clin, ccol);
#endif
      for(i = 0; (i < 4) && goOn; i++)
	{
	  lin = clin + delta[i][0];
	  col = ccol + delta[i][1];
	  if(check(lin, col))
	    {
	      if(!visited[lin][col])
		{
		  qlin[qlast] = lin;
		  qcol[qlast++] = col;
		  prevLin[lin][col] = clin;
		  prevCol[lin][col] = ccol;
		  visited[lin][col] = 1;
		  if(mazeCopy[lin][col] == '%') /* found it! */
		    {
#ifdef DEBUG
		      fprintf(stderr, "found cherry at <%d, %d>\n", lin, col);
#endif
		
		      goOn = 0;
		    }
		}
	    }
	}
    }
  for(i = 0; i < M; i++)
    for(j = 0; j < N; j++)
      visited[i][j] = 0;
  if(goOn)
    return 0; /* not found */
  
  mazeCopy[lin][col] = '.';
#ifdef DEBUG
  fprintf(stderr, "copying path\n");
#endif
  do
    {
#ifdef DEBUG
      fprintf(stderr, "copying cell <%d, %d>\n", lin, col);
#endif
      pLin[n] = lin; pCol[n++] = col;
      auxlin = prevLin[lin][col]; col = prevCol[lin][col]; lin = auxlin;
    }
  while(prevLin[lin][col] != -1);
  return n;
}

int findGreedyPath()
{
  int i, j, len = 1, lin = Plin, col = Pcol, n, cherries = 0;
  char pLin[10001], pCol[10001];

  pathlin[0] = lin;
  pathcol[0] = col;
  
  do
    {
#ifdef DEBUG
      fprintf(stderr, "calling findNearestCherry\n");
#endif
      n = findNearestCherry(lin, col, pLin, pCol);
#ifdef DEBUG
      fprintf(stderr, "the call to findNearestCherry completed\n");
#endif
      if(n) /* found */
	{
	  for(i = n - 1; (len <= T) && (i >= 0); i--)
	    {
	      pathlin[len] = pLin[i];
	      pathcol[len++] = pCol[i];
	    }
	  if(i < 0)
	    {
	      cherries++;
	      lin = pLin[0]; col = pCol[0]; 
	    }
	}
    }
  while((len <= T) && n);
  cherry_ = cherries;
  for(i = 0; i < len; i++)
    visited[pathlin[i]][pathcol[i]]++;
  if((len <= T) && cherries)
    { /* could not find a greedy path, but I have some cherries */
      /* generate randomly the rest */
      
      generatePath(len - 1, cherries, pathlin[len - 1], pathcol[len - 1]);
      len= T + 1;
    }
  if(len <= T)
    for(i = 0; i < len; i++)
      visited[pathlin[i]][pathcol[i]] = 0;
  
  return (len > T);
}

int main(int argc, char **argv)
{
  int i, maxcherry = 0, cherry, times = TIMES;

  clock_ = clock();
  goOn_ = 1;
  
  if(argc > 1)
    times = atoi(argv[1]);
  
  readMaze();
  /* printMaze(); */
  findP();
  srand(T * T + M * M + N * N);

  if((!check(Plin + 1, Pcol)) &&
     (!check(Plin - 1, Pcol)) &&
     (!check(Plin, Pcol + 1)) &&
     (!check(Plin, Pcol - 1)))
    {
      printf("No path!\n");
      exit(1);
    }

  if(!findGreedyPath())
      cherry = generatePath(0, 0, Plin, Pcol);
  climbPath();
  maxstep_ = step_;
  maxcherry = cherry_;
  generateMoves();
  strcpy(maxpath, path);
  cleanPath();
  
  while(goOn_)
    {
      cherry = generatePath(0, 0, Plin, Pcol);
      if(goOn_)
	{
	  climbPath();
	  if(goOn_)
	    {
	      cleanPath();
	      if(cherry_ > maxcherry)
		{
		  maxstep_ = step_;
		  maxcherry = cherry_;
		  generateMoves();
		  strcpy(maxpath, path);
		}
	    }
	}
    }

  fprintf(stderr, "number of cherries: %d, in %d climbs\n", maxcherry, maxstep_);
  printPath(maxpath, maxcherry);
  return 0;
}
